
#include <tizen.h>
#include <cairo.h>
#include "smwatch.h"

typedef struct appdata {
	Evas_Object *win;
	Evas_Object *conform;
	Evas_Object *image;
	Evas_Object *digit;
} appdata_s;

#define TEXT_BUF_SIZE 256

static void
update_watch(appdata_s *ad, watch_time_h watch_time, int ambient)
{
	char watch_text[TEXT_BUF_SIZE];
	int hour24, minute, second;

	if (watch_time == NULL)
		return;

	watch_time_get_hour24(watch_time, &hour24);
	watch_time_get_minute(watch_time, &minute);
	watch_time_get_second(watch_time, &second);

	static int s_hour24 = -1;
	static int s_minute = -1;

	if (s_hour24 != hour24 || s_minute != minute)
	{
		snprintf(watch_text, TEXT_BUF_SIZE, "%02d:%02d", hour24, minute);

		Evas_Object* obj = ad->digit;

		int width, height;
		evas_object_image_size_get(obj, &width, &height);

		unsigned char* buffer = (unsigned char*)evas_object_image_data_get (obj, 1);
		int stride = evas_object_image_stride_get (obj);

		cairo_surface_t *dst_surface = cairo_image_surface_create_for_data (buffer, CAIRO_FORMAT_ARGB32, width, height, stride);
		cairo_t *dst_cr = cairo_create (dst_surface);

		cairo_set_source_rgba(dst_cr, 0, 0, 0, 0);
		cairo_rectangle(dst_cr, 0, 0, width, height);
		cairo_set_operator(dst_cr, CAIRO_OPERATOR_SOURCE);
		cairo_fill(dst_cr);

		cairo_set_operator(dst_cr, CAIRO_OPERATOR_OVER);
		{
			// [SMGAL] Samsung internal font, Don't use this for further compatibility
			cairo_select_font_face(dst_cr, "SamsungOneFluid", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_NORMAL);

			cairo_set_font_size(dst_cr, 82);

			cairo_text_extents_t extents;
			cairo_text_extents(dst_cr, watch_text, &extents);

			cairo_set_source_rgba(dst_cr, 1, 1, 1, 1);
			cairo_move_to(dst_cr, (width - extents.width) / 2, height - (height - extents.height) / 2);
			cairo_show_text(dst_cr, watch_text);
		}

		cairo_surface_flush(dst_surface);

		evas_object_image_data_set(obj, cairo_image_surface_get_data(dst_surface));
		evas_object_image_data_update_add(obj, 0, 0, width, height);

		cairo_destroy(dst_cr);
		cairo_surface_destroy(dst_surface);

		s_hour24 = hour24;
		s_minute = minute;
	}
}

static void
create_base_gui(appdata_s *ad, int width, int height)
{
	int ret;
	watch_time_h watch_time = NULL;

	/* Window */
	ret = watch_app_get_elm_win(&ad->win);
	if (ret != APP_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to get window. err = %d", ret);
		return;
	}

	evas_object_resize(ad->win, width, height);

	/* Conformant */
	ad->conform = elm_conformant_add(ad->win);
	evas_object_size_hint_weight_set(ad->conform, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(ad->win, ad->conform);
	evas_object_show(ad->conform);

	{
		Evas_Object *img = evas_object_image_filled_add(evas_object_evas_get(ad->conform));

		char res_path[1024];
		char* res_path_buff = app_get_resource_path();
		sprintf(res_path, "%snico.png", res_path_buff);
		free(res_path_buff);

		evas_object_image_file_set(img, res_path, NULL);
		evas_object_move(img, 0, 0);
		evas_object_resize(img, 360, 360);
		evas_object_show(img);

		ad->image = img;
	}

	{
		Evas_Object *img = evas_object_image_filled_add(evas_object_evas_get(ad->conform));

		evas_object_image_size_set(img, 360, 80);
		evas_object_image_alpha_set(img, EINA_TRUE);
		evas_object_image_colorspace_set(img, EVAS_COLORSPACE_ARGB8888);
		evas_object_move(img, 0, 246);
		evas_object_resize(img, 360, 80);
		evas_object_show(img);

		ad->digit = img;
	}

	ret = watch_time_get_current_time(&watch_time);
	if (ret != APP_ERROR_NONE)
		dlog_print(DLOG_ERROR, LOG_TAG, "failed to get current time. err = %d", ret);

	update_watch(ad, watch_time, 0);
	watch_time_delete(watch_time);

	/* Show window after base gui is set up */
	evas_object_show(ad->win);
}

static bool
app_create(int width, int height, void *data)
{
	/* Hook to take necessary actions before main event loop starts
		Initialize UI resources and application's data
		If this function returns true, the main loop of application starts
		If this function returns false, the application is terminated */
	appdata_s *ad = data;

	create_base_gui(ad, width, height);

	return true;
}

static void
app_control(app_control_h app_control, void *data)
{
	/* Handle the launch request. */
}

static void
app_pause(void *data)
{
	/* Take necessary actions when application becomes invisible. */
}

static void
app_resume(void *data)
{
	/* Take necessary actions when application becomes visible. */
}

static void
app_terminate(void *data)
{
	/* Release all resources. */
}

static void
app_time_tick(watch_time_h watch_time, void *data)
{
	/* Called at each second while your app is visible. Update watch UI. */
	appdata_s *ad = data;
	update_watch(ad, watch_time, 0);
}

static void
app_ambient_tick(watch_time_h watch_time, void *data)
{
	/* Called at each minute while the device is in ambient mode. Update watch UI. */
	appdata_s *ad = data;
	update_watch(ad, watch_time, 1);
}

static void
app_ambient_changed(bool ambient_mode, void *data)
{
	appdata_s *ad = data;

	if (ambient_mode)
	{
		evas_object_hide(ad->image);
		dlog_print(DLOG_INFO, "[SMGAL]", "evas_object_hide()");
	}
	else
	{
		evas_object_show(ad->image);
		dlog_print(DLOG_INFO, "[SMGAL]", "evas_object_show()");
	}
}

static void
watch_app_lang_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_LANGUAGE_CHANGED*/
	char *locale = NULL;
	app_event_get_language(event_info, &locale);
	elm_language_set(locale);
	free(locale);
	return;
}

static void
watch_app_region_changed(app_event_info_h event_info, void *user_data)
{
	/*APP_EVENT_REGION_FORMAT_CHANGED*/
}

int
main(int argc, char *argv[])
{
	dlog_print(DLOG_ERROR, LOG_TAG, "main()");

	appdata_s ad = {0,};
	int ret = 0;

	watch_app_lifecycle_callback_s event_callback = {0,};
	app_event_handler_h handlers[5] = {NULL, };

	event_callback.create = app_create;
	event_callback.terminate = app_terminate;
	event_callback.pause = app_pause;
	event_callback.resume = app_resume;
	event_callback.app_control = app_control;
	event_callback.time_tick = app_time_tick;
	event_callback.ambient_tick = app_ambient_tick;
	event_callback.ambient_changed = app_ambient_changed;

	watch_app_add_event_handler(&handlers[APP_EVENT_LANGUAGE_CHANGED],
		APP_EVENT_LANGUAGE_CHANGED, watch_app_lang_changed, &ad);
	watch_app_add_event_handler(&handlers[APP_EVENT_REGION_FORMAT_CHANGED],
		APP_EVENT_REGION_FORMAT_CHANGED, watch_app_region_changed, &ad);

	ret = watch_app_main(argc, argv, &event_callback, &ad);
	if (ret != APP_ERROR_NONE) {
		dlog_print(DLOG_ERROR, LOG_TAG, "watch_app_main() is failed. err = %d", ret);
	}

	return ret;
}

